@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,17 +16,20 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core.client_options import ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
-OptionalRetry = Union[retries.Retry, object]
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
 
 from google.cloud.iap_v1.types import service
 from google.iam.v1 import iam_policy_pb2  # type: ignore
@@ -111,6 +114,42 @@ class IdentityAwareProxyAdminServiceAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return IdentityAwareProxyAdminServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> IdentityAwareProxyAdminServiceTransport:
         """Returns the transport used by the client instance.
@@ -185,6 +224,26 @@ class IdentityAwareProxyAdminServiceAsyncClient:
         information about managing access via IAP can be found at:
         https://cloud.google.com/iap/docs/managing-access#managing_access_via_the_api
 
+
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_set_iam_policy():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyAdminServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.SetIamPolicyRequest(
+                    resource="resource_value",
+                )
+
+                # Make the request
+                response = client.set_iam_policy(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.iam.v1.iam_policy_pb2.SetIamPolicyRequest, dict]):
                 The request object. Request message for `SetIamPolicy`
@@ -293,6 +352,26 @@ class IdentityAwareProxyAdminServiceAsyncClient:
         IAP can be found at:
         https://cloud.google.com/iap/docs/managing-access#managing_access_via_the_api
 
+
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_get_iam_policy():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyAdminServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.GetIamPolicyRequest(
+                    resource="resource_value",
+                )
+
+                # Make the request
+                response = client.get_iam_policy(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.iam.v1.iam_policy_pb2.GetIamPolicyRequest, dict]):
                 The request object. Request message for `GetIamPolicy`
@@ -401,6 +480,27 @@ class IdentityAwareProxyAdminServiceAsyncClient:
         via IAP can be found at:
         https://cloud.google.com/iap/docs/managing-access#managing_access_via_the_api
 
+
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_test_iam_permissions():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyAdminServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.TestIamPermissionsRequest(
+                    resource="resource_value",
+                    permissions=['permissions_value_1', 'permissions_value_2'],
+                )
+
+                # Make the request
+                response = client.test_iam_permissions(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.iam.v1.iam_policy_pb2.TestIamPermissionsRequest, dict]):
                 The request object. Request message for
@@ -452,6 +552,26 @@ class IdentityAwareProxyAdminServiceAsyncClient:
         r"""Gets the IAP settings on a particular IAP protected
         resource.
 
+
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_get_iap_settings():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyAdminServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.GetIapSettingsRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_iap_settings(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.iap_v1.types.GetIapSettingsRequest, dict]):
                 The request object. The request sent to GetIapSettings.
@@ -499,6 +619,29 @@ class IdentityAwareProxyAdminServiceAsyncClient:
         r"""Updates the IAP settings on a particular IAP protected resource.
         It replaces all fields unless the ``update_mask`` is set.
 
+
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_update_iap_settings():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyAdminServiceClient()
+
+                # Initialize request argument(s)
+                iap_settings = iap_v1.IapSettings()
+                iap_settings.name = "name_value"
+
+                request = iap_v1.UpdateIapSettingsRequest(
+                    iap_settings=iap_settings,
+                )
+
+                # Make the request
+                response = client.update_iap_settings(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.iap_v1.types.UpdateIapSettingsRequest, dict]):
                 The request object. The request sent to
