@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,17 +16,20 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core.client_options import ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
-OptionalRetry = Union[retries.Retry, object]
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
 
 from google.cloud.iap_v1.services.identity_aware_proxy_o_auth_service import pagers
 from google.cloud.iap_v1.types import service
@@ -113,6 +116,42 @@ class IdentityAwareProxyOAuthServiceAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return IdentityAwareProxyOAuthServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> IdentityAwareProxyOAuthServiceTransport:
         """Returns the transport used by the client instance.
@@ -184,6 +223,25 @@ class IdentityAwareProxyOAuthServiceAsyncClient:
     ) -> service.ListBrandsResponse:
         r"""Lists the existing brands for the project.
 
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_list_brands():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyOAuthServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.ListBrandsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                response = client.list_brands(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.iap_v1.types.ListBrandsRequest, dict]):
                 The request object. The request sent to ListBrands.
@@ -239,6 +297,26 @@ class IdentityAwareProxyOAuthServiceAsyncClient:
         project, and that the specified support email is owned
         by the caller.
 
+
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_create_brand():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyOAuthServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.CreateBrandRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                response = client.create_brand(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.iap_v1.types.CreateBrandRequest, dict]):
                 The request object. The request sent to CreateBrand.
@@ -288,6 +366,25 @@ class IdentityAwareProxyOAuthServiceAsyncClient:
     ) -> service.Brand:
         r"""Retrieves the OAuth brand of the project.
 
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_get_brand():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyOAuthServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.GetBrandRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_brand(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.iap_v1.types.GetBrandRequest, dict]):
                 The request object. The request sent to GetBrand.
@@ -340,6 +437,26 @@ class IdentityAwareProxyOAuthServiceAsyncClient:
         the project exists and that it is set for internal-only
         use.
 
+
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_create_identity_aware_proxy_client():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyOAuthServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.CreateIdentityAwareProxyClientRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                response = client.create_identity_aware_proxy_client(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.iap_v1.types.CreateIdentityAwareProxyClientRequest, dict]):
                 The request object. The request sent to
@@ -389,6 +506,26 @@ class IdentityAwareProxyOAuthServiceAsyncClient:
     ) -> pagers.ListIdentityAwareProxyClientsAsyncPager:
         r"""Lists the existing clients for the brand.
 
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_list_identity_aware_proxy_clients():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyOAuthServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.ListIdentityAwareProxyClientsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_identity_aware_proxy_clients(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.iap_v1.types.ListIdentityAwareProxyClientsRequest, dict]):
                 The request object. The request sent to
@@ -448,6 +585,26 @@ class IdentityAwareProxyOAuthServiceAsyncClient:
         r"""Retrieves an Identity Aware Proxy (IAP) OAuth client.
         Requires that the client is owned by IAP.
 
+
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_get_identity_aware_proxy_client():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyOAuthServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.GetIdentityAwareProxyClientRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_identity_aware_proxy_client(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.iap_v1.types.GetIdentityAwareProxyClientRequest, dict]):
                 The request object. The request sent to
@@ -499,6 +656,26 @@ class IdentityAwareProxyOAuthServiceAsyncClient:
         secret. Useful if the secret was compromised. Requires
         that the client is owned by IAP.
 
+
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_reset_identity_aware_proxy_client_secret():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyOAuthServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.ResetIdentityAwareProxyClientSecretRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.reset_identity_aware_proxy_client_secret(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.iap_v1.types.ResetIdentityAwareProxyClientSecretRequest, dict]):
                 The request object. The request sent to
@@ -551,6 +728,23 @@ class IdentityAwareProxyOAuthServiceAsyncClient:
         number of clients in a given project, and cleaning up
         after tests. Requires that the client is owned by IAP.
 
+
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_delete_identity_aware_proxy_client():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyOAuthServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.DeleteIdentityAwareProxyClientRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                client.delete_identity_aware_proxy_client(request=request)
+
         Args:
             request (Union[google.cloud.iap_v1.types.DeleteIdentityAwareProxyClientRequest, dict]):
                 The request object. The request sent to
