@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,23 +14,25 @@
 # limitations under the License.
 #
 from collections import OrderedDict
-from distutils import util
 import os
 import re
 from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core import client_options as client_options_lib  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core import client_options as client_options_lib
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport import mtls  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 from google.auth.exceptions import MutualTLSChannelError  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
-OptionalRetry = Union[retries.Retry, object]
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
 
 from google.cloud.iap_v1.services.identity_aware_proxy_o_auth_service import pagers
 from google.cloud.iap_v1.types import service
@@ -228,6 +230,73 @@ class IdentityAwareProxyOAuthServiceClient(
         m = re.match(r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)$", path)
         return m.groupdict() if m else {}
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[client_options_lib.ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        if client_options is None:
+            client_options = client_options_lib.ClientOptions()
+        use_client_cert = os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false")
+        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
+        if use_client_cert not in ("true", "false"):
+            raise ValueError(
+                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
+            )
+        if use_mtls_endpoint not in ("auto", "never", "always"):
+            raise MutualTLSChannelError(
+                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
+            )
+
+        # Figure out the client cert source to use.
+        client_cert_source = None
+        if use_client_cert == "true":
+            if client_options.client_cert_source:
+                client_cert_source = client_options.client_cert_source
+            elif mtls.has_default_client_cert_source():
+                client_cert_source = mtls.default_client_cert_source()
+
+        # Figure out which api endpoint to use.
+        if client_options.api_endpoint is not None:
+            api_endpoint = client_options.api_endpoint
+        elif use_mtls_endpoint == "always" or (
+            use_mtls_endpoint == "auto" and client_cert_source
+        ):
+            api_endpoint = cls.DEFAULT_MTLS_ENDPOINT
+        else:
+            api_endpoint = cls.DEFAULT_ENDPOINT
+
+        return api_endpoint, client_cert_source
+
     def __init__(
         self,
         *,
@@ -278,50 +347,22 @@ class IdentityAwareProxyOAuthServiceClient(
         if client_options is None:
             client_options = client_options_lib.ClientOptions()
 
-        # Create SSL credentials for mutual TLS if needed.
-        use_client_cert = bool(
-            util.strtobool(os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false"))
+        api_endpoint, client_cert_source_func = self.get_mtls_endpoint_and_cert_source(
+            client_options
         )
 
-        client_cert_source_func = None
-        is_mtls = False
-        if use_client_cert:
-            if client_options.client_cert_source:
-                is_mtls = True
-                client_cert_source_func = client_options.client_cert_source
-            else:
-                is_mtls = mtls.has_default_client_cert_source()
-                if is_mtls:
-                    client_cert_source_func = mtls.default_client_cert_source()
-                else:
-                    client_cert_source_func = None
-
-        # Figure out which api endpoint to use.
-        if client_options.api_endpoint is not None:
-            api_endpoint = client_options.api_endpoint
-        else:
-            use_mtls_env = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
-            if use_mtls_env == "never":
-                api_endpoint = self.DEFAULT_ENDPOINT
-            elif use_mtls_env == "always":
-                api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-            elif use_mtls_env == "auto":
-                if is_mtls:
-                    api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-                else:
-                    api_endpoint = self.DEFAULT_ENDPOINT
-            else:
-                raise MutualTLSChannelError(
-                    "Unsupported GOOGLE_API_USE_MTLS_ENDPOINT value. Accepted "
-                    "values: never, auto, always"
-                )
+        api_key_value = getattr(client_options, "api_key", None)
+        if api_key_value and credentials:
+            raise ValueError(
+                "client_options.api_key and credentials are mutually exclusive"
+            )
 
         # Save or instantiate the transport.
         # Ordinarily, we provide the transport, but allowing a custom transport
         # instance provides an extensibility point for unusual situations.
         if isinstance(transport, IdentityAwareProxyOAuthServiceTransport):
             # transport is a IdentityAwareProxyOAuthServiceTransport instance.
-            if credentials or client_options.credentials_file:
+            if credentials or client_options.credentials_file or api_key_value:
                 raise ValueError(
                     "When providing a transport instance, "
                     "provide its credentials directly."
@@ -333,6 +374,15 @@ class IdentityAwareProxyOAuthServiceClient(
                 )
             self._transport = transport
         else:
+            import google.auth._default  # type: ignore
+
+            if api_key_value and hasattr(
+                google.auth._default, "get_api_key_credentials"
+            ):
+                credentials = google.auth._default.get_api_key_credentials(
+                    api_key_value
+                )
+
             Transport = type(self).get_transport_class(transport)
             self._transport = Transport(
                 credentials=credentials,
@@ -355,6 +405,25 @@ class IdentityAwareProxyOAuthServiceClient(
     ) -> service.ListBrandsResponse:
         r"""Lists the existing brands for the project.
 
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_list_brands():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyOAuthServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.ListBrandsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                response = client.list_brands(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.iap_v1.types.ListBrandsRequest, dict]):
                 The request object. The request sent to ListBrands.
@@ -411,6 +480,26 @@ class IdentityAwareProxyOAuthServiceClient(
         project, and that the specified support email is owned
         by the caller.
 
+
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_create_brand():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyOAuthServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.CreateBrandRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                response = client.create_brand(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.iap_v1.types.CreateBrandRequest, dict]):
                 The request object. The request sent to CreateBrand.
@@ -461,6 +550,25 @@ class IdentityAwareProxyOAuthServiceClient(
     ) -> service.Brand:
         r"""Retrieves the OAuth brand of the project.
 
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_get_brand():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyOAuthServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.GetBrandRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_brand(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.iap_v1.types.GetBrandRequest, dict]):
                 The request object. The request sent to GetBrand.
@@ -514,6 +622,26 @@ class IdentityAwareProxyOAuthServiceClient(
         the project exists and that it is set for internal-only
         use.
 
+
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_create_identity_aware_proxy_client():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyOAuthServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.CreateIdentityAwareProxyClientRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                response = client.create_identity_aware_proxy_client(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.iap_v1.types.CreateIdentityAwareProxyClientRequest, dict]):
                 The request object. The request sent to
@@ -566,6 +694,26 @@ class IdentityAwareProxyOAuthServiceClient(
     ) -> pagers.ListIdentityAwareProxyClientsPager:
         r"""Lists the existing clients for the brand.
 
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_list_identity_aware_proxy_clients():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyOAuthServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.ListIdentityAwareProxyClientsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_identity_aware_proxy_clients(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.iap_v1.types.ListIdentityAwareProxyClientsRequest, dict]):
                 The request object. The request sent to
@@ -628,6 +776,26 @@ class IdentityAwareProxyOAuthServiceClient(
         r"""Retrieves an Identity Aware Proxy (IAP) OAuth client.
         Requires that the client is owned by IAP.
 
+
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_get_identity_aware_proxy_client():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyOAuthServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.GetIdentityAwareProxyClientRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_identity_aware_proxy_client(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.iap_v1.types.GetIdentityAwareProxyClientRequest, dict]):
                 The request object. The request sent to
@@ -682,6 +850,26 @@ class IdentityAwareProxyOAuthServiceClient(
         secret. Useful if the secret was compromised. Requires
         that the client is owned by IAP.
 
+
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_reset_identity_aware_proxy_client_secret():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyOAuthServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.ResetIdentityAwareProxyClientSecretRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.reset_identity_aware_proxy_client_secret(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.iap_v1.types.ResetIdentityAwareProxyClientSecretRequest, dict]):
                 The request object. The request sent to
@@ -737,6 +925,23 @@ class IdentityAwareProxyOAuthServiceClient(
         number of clients in a given project, and cleaning up
         after tests. Requires that the client is owned by IAP.
 
+
+        .. code-block:: python
+
+            from google.cloud import iap_v1
+
+            def sample_delete_identity_aware_proxy_client():
+                # Create a client
+                client = iap_v1.IdentityAwareProxyOAuthServiceClient()
+
+                # Initialize request argument(s)
+                request = iap_v1.DeleteIdentityAwareProxyClientRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                client.delete_identity_aware_proxy_client(request=request)
+
         Args:
             request (Union[google.cloud.iap_v1.types.DeleteIdentityAwareProxyClientRequest, dict]):
                 The request object. The request sent to
